@@ -1,4 +1,4 @@
-static char rcsid[] = "$Id: colon.c,v 1.28 2000/02/07 17:22:06 will Exp $";
+static const char rcsid[] = "$Id: colon.c,v 1.29 2004/04/30 17:00:58 will Exp $";
 
 #include "super.h"
 #include "version.h"
@@ -120,7 +120,6 @@ global_arg(word)
 char *word;	/* opt=xxx or condition~Pattern or PermittedUserPattern */
 {
     /* Return 0 on success, -1 if formatting error. */
-    extern int error_syslog;
     int invert, iscondition;
     char **glob;
     char *s;
@@ -270,9 +269,11 @@ colon_die(command)
 char *command;
 {
     char *text;
+#ifdef COLLECT_ALL_TOKENS
     extern char *strqS_start;
+#endif    
 
-#if 1
+#ifndef COLLECT_ALL_TOKENS
     /* This collects next token only */
     text = strqtokS(NULL, SEP, NULL, NULL, 1);
 #else
@@ -294,10 +295,8 @@ int
 colon_getenv(command)
 char *command;
 {
-    extern char *strqS_start;
-    char *varname, *varbody;
-    char *s, *defn;
-    int l;
+    char *varname;
+    char  *defn;
 
     for ( ; (varname = strqtokS(NULL, SEP, NULL, NULL, 1)) ; ) {
 	defn = getenv(varname);
@@ -326,8 +325,7 @@ colon_if(command, colonif_succeeded)
 char *command;
 int *colonif_succeeded;
 {
-    char *left, *op, *right, *s;
-    extern char *strqS_start;
+    char *left, *op, *right;
 
     left = strqtokS(NULL, SEP, NULL, NULL, 1);
     if (!left)
