@@ -1,4 +1,4 @@
-static const char rcsid[] = "$Id: options.c,v 1.118 2004/04/30 17:00:58 will Exp $";
+static const char rcsid[] = "$Id: options.c,v 1.129 2004/10/22 00:22:19 will Exp $";
 /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
 /* Process a global xxx=yyy option */
 /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
@@ -13,7 +13,7 @@ static const char rcsid[] = "$Id: options.c,v 1.118 2004/04/30 17:00:58 will Exp
 Option opts[] = {
 	{ "DIE",		LOCAL,		option_die },	/* OBSOLETE */
 	{ "addgroups",		LIST|GLOBAL|LOCAL,	option_groups },
-	{ "arg",		LOCAL,		option_arg },
+	{ "arg",		GLOBAL|LOCAL,	option_arg },
 	{ "argv0",		LOCAL,		option_argv0 },
 	{ "auth",		GLOBAL|LOCAL,	option_auth },
 	{ "authprompt",		GLOBAL|LOCAL,	option_authprompt },
@@ -38,7 +38,7 @@ Option opts[] = {
 	{ "mailany",		GLOBAL|LOCAL,	option_mailany },
 	{ "maxenvlen",		GLOBAL|LOCAL,	option_maxenvlen },
 	{ "maxlen",		GLOBAL|LOCAL,	option_maxlen },
-	{ "nargs",		LOCAL,		option_nargs },
+	{ "nargs",		GLOBAL|LOCAL,	option_nargs },
 	{ "nice",		GLOBAL|LOCAL,	option_nice },
 	{ "owner",		GLOBAL|LOCAL,	option_owner },
 	{ "password",		GLOBAL|LOCAL,	option_password },
@@ -325,8 +325,19 @@ option_local_clear_settings()
     free_Simple2List(&localinfo.userpats);
     free_SimpleList(&localinfo.origtext);
 
-    localinfo.usr_args[0] = localinfo.usr_args[1] = -1;
-    StrEltsUnused(&localinfo.argpats);
+    localinfo.usr_args[0] = globalinfo.usr_args[0];
+    localinfo.usr_args[1] = globalinfo.usr_args[1];
+
+    /* special case: don't initialize localinfo argpats to be
+     * a copy of globalinfo argpats.  That's because the
+     * argpats accumulate, and we mustn't accumulate a pile
+     * of local options on top of a copy of the global ones.
+     * When we need to use them, we'll copy from global value
+     * if the local value is still a null ptr.
+     */
+    ARfree(localinfo.argpats.next);
+    localinfo.argpats.next = NULL;
+
     if (localinfo.checkvar)
 	blkfree(localinfo.checkvar);
     localinfo.checkvar = NULL;
@@ -479,14 +490,17 @@ int isglobal;
     /* Must be argMMM-NNN<SEP>SSS or argNNN<SEP>SSS */
     int i, iarg1, iarg2;
     char eq;
+    ArgRangePat *argpats;
 
     if (debug)
 	fprintf(stderr, "\toption:looks like argMMM-NNN=%s (%s)...\n",
 				pat, isglobal ? "global" : "local");
 
-    if (*pat == '\0')
-	return Error(0, 0,
-		"%t\n\tImproper use of argNNN%cXXX option\n", OPTION_SEP);
+    if (isglobal) {
+	argpats = &globalinfo.argpats;
+    } else {
+	argpats = &localinfo.argpats;
+    }
 
     /* Check that word matches argNNN=SSS */
     i = sscanf(word, "arg%d-%d%c", &iarg1, &iarg2, &eq);
@@ -507,16 +521,14 @@ int isglobal;
     /* Put pattern into the argpat list. */
     {
 	char buf[500];
-	int iarg;
 	if (strlen(pat) > sizeof(buf)-3)
 	    return Error(0, 0,
 			    "%t\n\tArgument pattern <%s> too long (max %d)\n",
 			    pat, sizeof(buf)-3);
 	anchor(pat, buf);
-	for (iarg = iarg1; iarg <= iarg2; iarg++) {
-	    if (StrEltCpy(&localinfo.argpats, iarg, buf) == -1)
-		return Error(0, 0,
-		    "Failed to malloc space for argument pattern #%d\n", iarg);
+	if (ARinsert(argpats, iarg1, iarg2, buf) == -1) {
+	    return Error(0, 0,
+		"Failed to malloc space for argument pattern <%s>\n", buf);
 	}
     }
     return 0; 
@@ -968,17 +980,23 @@ char *s;
 int isglobal;
 {
     int i, m, n;
+    int *nargsp;
 
     if (debug)
 	fprintf(stderr, "\toption:nargs=%s (%s)\n",
 				s, isglobal ? "global" : "local");
 
+    if (isglobal) {
+	nargsp = globalinfo.usr_args;
+    } else {
+	nargsp = localinfo.usr_args;
+    }
     i = sscanf(s, "%u-%u", &m, &n);
     switch(i) {
-    case 1:     localinfo.usr_args[0] = localinfo.usr_args[1] = m;
+    case 1:     nargsp[0] = nargsp[1] = m;
 		break;
-    case 2:     localinfo.usr_args[0] = m;
-		localinfo.usr_args[1] = n;
+    case 2:     nargsp[0] = m;
+		nargsp[1] = n;
 		break;
     default:
 	return Error(0, 0,
@@ -1098,36 +1116,36 @@ password checking -- sorry.\n");
 }
 
 static int
-option_patterns(word, s, isglobal)
+option_patterns(word, pat_name, isglobal)
 char *word;
-char *s;
+char *pat_name;
 int isglobal;
 {
     char *p;
     if (debug)
 	fprintf(stderr, "\toption:patterns=%s (%s)\n",
-				s, isglobal ? "global" : "local");
+				pat_name, isglobal ? "global" : "local");
 
-    if (strcmp(s, "shell") == 0) {
+    if (strcmp(pat_name, "shell") == 0) {
 	pat_compile = shell_compile;
 	pat_compare = shell_compare;
 	need_re_anchor = 0;
-    } else if (strcmp(s, "regex") == 0) {
+    } else if (strcmp(pat_name, "regex") == 0) {
 	pat_compile = s_re_comp;
 	pat_compare = s_re_exec;
 	need_re_anchor = 1;
 #ifdef HAVE_POSIX_REGEX
-    } else if (strncmp(s, "posix", 5) == 0 &&
-			(s[5] == '/' || strlen(s) == 5)) {
-	char *buf = strdup(s);
+    } else if (strncmp(pat_name, "posix", 5) == 0 &&
+		(pat_name[5] == '/' || strlen(pat_name) == 5)) {
+	char *buf = strdup(pat_name);
 	if (!buf) {
 	    return Error(0, 0,
-			"%t\n\tCan't malloc space for copy of <%s>\n", s);
+		"%t\n\tCan't malloc space for copy of <%s>\n", pat_name);
 	}
 	/* Check the options: we allow:
 	 *	posix[/extended][/icase]
 	 */
-	p = strtok(s, "/");
+	p = strtok(buf, "/");
 	if (strcmp(p, "posix") != 0) {
 	    return Error(0, 0, "%t\n\tInternal error: got pat type <%s> when altready checked it's simply `posix'!\n", p);
 	}
@@ -1149,12 +1167,13 @@ int isglobal;
     } else {
 #ifdef HAVE_POSIX_REGEX
 	return Error(0, 0, "%t\n\tInvalid pattern type `%s'.  \
-Valid: \"shell\", \"regex\", and \"posix[/extended][/icase]\"\n", s);
+Valid: \"shell\", \"regex\", and \"posix[/extended][/icase]\"\n", pat_name);
 #else
 	return Error(0, 0, "%t\n\tInvalid pattern type `%s'.  \
-Valid: \"shell\" and \"regex\"\n", s);
+Valid: \"shell\" and \"regex\"\n", pat_name);
 #endif
     }
+    add_variable("PATTERNS", pat_name);
     return 0;
 }
 
