@@ -254,6 +254,14 @@ struct strArray {
 };
 typedef struct strArray StrArray;
 
+struct argRangePat {
+    char *pat;		/* pattern to be matched by ... */
+    int arg1;		/* this arg through... */
+    int arg2;		/* this arg. */
+    struct argRangePat *next;
+};
+typedef struct argRangePat ArgRangePat;
+
 struct ourTime {
     time_t start;	/* when program started */
     short min;		/* local time of day, in minutes */
@@ -351,6 +359,8 @@ struct globalInfo {
 
     long maxlen1arg;	/* max len of any single arg */
     long maxlenargs;	/* max len of all args, combined */
+    int usr_args[2];	/* number of user-entered args allowed */
+    ArgRangePat argpats;	/* arg[MMM-]NNN=xxx arguments */
 
     AuthInfo authinfo;	/* authentication information */
 
@@ -439,7 +449,7 @@ struct localInfo {
     int usr_args[2];	/* number of user-entered args allowed */
     long maxlen1arg;	/* max len of any single arg */
     long maxlenargs;	/* max len of all args, combined */
-    StrArray argpats;	/* argNNN=xxx arguments */
+    ArgRangePat argpats;	/* arg[MMM-]NNN=xxx arguments */
     char **checkvar;	/* null-term'd list of vars from checkvar=var[,...] */
 
     char mailcmd[500];	/* Information for logging via mail */
@@ -454,6 +464,11 @@ typedef struct localInfo LocalInfo;
 
 extern FileList *currfile;		/* list of currently-open files */
 
+extern char authInitMsg1[];		/* msg from authentication init */
+extern char authInitMsg2[];		/* suppl msg from authentication init */
+extern int authInitErrno;		/* errno, if relevant, to go
+					 * with authInitMsg1 */
+
 extern GlobalInfo globalinfo;		/* :global info */
 
 extern UserInfo userinfo;		/* User's info */
@@ -463,6 +478,10 @@ extern LocalInfo localinfo;		/* per-cmd info */
 extern Conditions matches;		/* To keep track of what matched */
 
 /* ========================================================================= */
+void	 ARfree P__((ArgRangePat *head));
+int	ARinsert P__((ArgRangePat *start, int arg1, int arg2, char *pat));
+ArgRangePat *
+	ARnext P__((ArgRangePat *start, int iarg));
 void	add_builtin_variables P__(( void ));
 #ifdef HAVE_SYSINFO
 int	add_sysinfo_variables P__(( void ));
@@ -553,8 +572,8 @@ int	InsertCondition P__(( char *, char *, int ));
 int	InsertTimeList P__(( char *, char **, TimeList *, char *, int ));
 int	InsertUserList P__((char *, char **, Simple2List *, SimpleList *, int));
 void	logmsg P__(( char * cmd, char ** args ));
-int	makedir P__(( char *directories ));
-char*	makedirname P__(( char *prefix, char *hostname, char *path ));
+int	makedir P__(( char *directories, int *err, char *msgbuf ));
+char*	makedirname P__(( char *prefix, char *hostname, char *path, int *err, char *msgbuf ));
 int	match_pattern P__((int match, int glob, char *str, char *pattern));
 void	match_ugh_user P__((Simple2List *sl, int isglobal));
 void	matchtime P__(( OurTime *our, TimeList *tl ));
@@ -593,6 +612,7 @@ char*	shell_compile P__(( char *));
 void	store_nice_incr P__((int nice_incr, int is_global));
 void	store_umask P__((int mask, int is_global));
 char*	str_val P__(( char *left, int sep, char *str ));
+int	StrBulkCpy P__(( StrArray *dst, int dst_ielt, StrArray *src, int src_ielt, int nelt ));
 int	StrEltCpy P__(( StrArray *a, int ielt, char *str));
 char*	StrEltGetPtr P__(( StrArray *a, int ielt ));
 void	StrEltsUnused P__(( StrArray *a ));
