@@ -1,4 +1,4 @@
-static const char rcsid[] = "$Id: utils.c,v 1.126 2004/04/30 17:00:58 will Exp $";
+static const char rcsid[] = "$Id: utils.c,v 1.153 2004/10/22 00:26:16 will Exp $";
 #include "super.h"
 
 /* Variables table flag -- !0 means has been created */
@@ -764,7 +764,7 @@ StrArray *a;
 /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
 char *
 StrEltGetPtr(a, ielt)
-StrArray *a;	/* StrArray with element to fill */
+StrArray *a;	/* StrArray with element of interest */
 int ielt;	/* Element to return */
     /* Returns NULL if no such element (or if unused) */
 {
@@ -783,23 +783,30 @@ int
 StrEltCpy(a, ielt, str)
 StrArray *a;	/* StrArray with element to fill */
 int ielt;	/* Element to be filled */
-char *str;	/* what to put into ielt'th place */
+char *str;	/* what to put into ielt'th place; if NULL, elt is "unused". */
 {
-    int l = strlen(str);
+    int l;
+    l = (str) ? strlen(str) : 0;
+
     if (StrNalloc(a, ielt) == -1)
 	return -1;
 
     a->str[ielt].used = 1;
     if (a->str[ielt].n == 0) {
 	a->str[ielt].s = malloc(l+1);
-	a->str[ielt].n = l+1;
     } else if (a->str[ielt].n < l+1) {
 	a->str[ielt].s = realloc(a->str[ielt].s, l+1);
 	a->str[ielt].n = l+1;
     }
-    if (!a->str[ielt].s)
+    if (!a->str[ielt].s) {
+	a->str[ielt].n = 0;
 	return -1;
-    strcpy(a->str[ielt].s, str);
+    }
+    if (str) {
+	strcpy(a->str[ielt].s, str ? str : "");
+    } else {
+	a->str[ielt].used = 0;
+    }
     return 0;
 }
 
@@ -814,17 +821,17 @@ StrArray *a;
 }
 
 /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
-/* Return number of in-use elements of StrArray */
+/* Return last index of in-use elements of StrArray */
 /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
 int
 StrLastInUse(a)
-StrArray *a;	/* StrArray with element to fill */
+StrArray *a;
 {
     int i;
     int last;
 
     for (i=0, last = -1; i < a->n; i++) {
-	if (a->str[i].used)
+	if (a->str[i].n && a->str[i].used)
 	    last = i;
     }
 
@@ -875,6 +882,123 @@ int ielt;
     return 0;
 }
 
+/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
+/* Copy StrArray src to StrArray dst, starting at element src_ielt
+ * in the src array, and at element dst_ielt in the dst array;
+ * copy nelt elements.
+ * If there are fewer than (src_ielt + nelt) elements, silently stop
+ * copying at the end of the src array.
+ * Returns -1 on malloc/realloc failure, 0 otherwise.
+ */
+/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
+int
+StrBulkCpy(dst, dst_ielt, src, src_ielt, nelt)
+StrArray *dst;	/* dstination array */
+int dst_ielt;	/* starting element to fill */
+StrArray *src;	/* sourced array */
+int src_ielt;	/* starting element to copy from */
+int nelt;	/* number of elements to copy */
+{
+    int i;
+    char *s;
+
+    if ((nelt + src_ielt) > StrNElts(src)) {
+	/* not that many src elements */
+	nelt = StrNElts(src) - src_ielt;
+    }
+    /* make sure dst has enough space */
+    if (StrNalloc(dst, (dst_ielt + nelt)) != 0) {
+	return -1;	/* can't make dst sufficiently large! */
+    }
+
+    for (i = 0; i < nelt; i++) {
+	s = StrEltGetPtr(src, i + src_ielt);
+	if (StrEltCpy(dst, i + dst_ielt, s) != 0) {
+	    return -1;	/* copy failed */
+	}
+    }
+    return 0;
+}
+
+/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
+/*
+ * Free everything in an ArgRangePat list.
+ * Don't free the pointer to the head.
+ */
+/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
+void
+ARfree(
+    ArgRangePat *head)
+{
+    ArgRangePat *arp, *next;
+    if (!head) {
+	return;
+    }
+
+    for (arp = head->next ; arp; arp = next) {
+	next = arp->next;
+	if (arp->pat) {
+	    free(arp->pat);
+	}
+	free(arp);
+    }
+}
+
+/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
+/*
+ * Insert a new string at the head of an arglist.
+ * Return 0 on success, -1 on malloc failure.
+ */
+/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
+int
+ARinsert(
+    ArgRangePat *head,
+    int arg1,
+    int arg2,
+    char *pat)
+{
+    ArgRangePat *arp;
+    arp = malloc(sizeof(ArgRangePat));
+    if (!arp) {
+	return -1;
+    }
+    arp->pat = malloc(strlen(pat)+1);
+    if (!arp->pat) {
+	return -1;
+    }
+    strcpy(arp->pat, pat);
+    arp->arg1 = arg1;
+    arp->arg2 = arg2;
+    arp->next = head->next;
+    head->next = arp;
+    return 0;
+}
+
+/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
+/*
+ * Walk an ArgRangePat list, finding the next pattern that applies
+ * to the iarg'th argument.  If the start'th argument applies,
+ * return it.  Stop if we reach an empty pattern.
+ */
+/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
+ArgRangePat *
+ARnext(
+    ArgRangePat *start,
+    int iarg)
+{
+    ArgRangePat *arp;
+    for (arp=start; arp; arp = arp->next) {
+	if (!arp->pat || !arp->pat[0]) {
+	    /* Stop if pattern is empty */
+	    return NULL;
+	} else if (arp->arg1 <= iarg  &&  iarg <= arp->arg2) {
+	    return arp;
+	}
+    }
+    /* input was null ptr, or ran out of list */
+    return NULL;
+}
+
 /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
 /* Copy up to n-1 characters into "to"; then null-terminate.
  * Return 1 if all characters fitted into "to", else return 0.
@@ -1051,22 +1175,40 @@ expected logger pid = %d; waiting for correct pid...\n",
      * internet domains, since it is frequently not a
      * fully-qualified domain name.  Therefore you should NOT
      * share the timestamp directory outside the local domain.
+     *
+
+     * *err is returned with an errno value, if there was an error
+     * for which there's a relevant errno.  Otherwise it's set to 0.
+     *
+     * If msgbuf is non-null pointer, then it's *assumed* long enough
+     * for error message (1000 chars is enough), and error messages
+     * are printed into it; otherwise, error messages are Error()'d.
+     *
      */
 
     /* returns NULL on error, constructed path otherwise */
 /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
 char *
-makedirname(prefix, hostname, file)
+makedirname(prefix, hostname, file, err, msgbuf)
 char *prefix, *hostname, *file;
+int *err;
+char *msgbuf;
 {
     int l;
     char *s, str[MAXPATHLEN];
 
+    *err = 0;
     l = strlen(prefix) + 1 + strlen(hostname) + 1;
     if (l >= MAXPATHLEN) {
-	Error(1, 0,
+	if (msgbuf) {
+	    sprintf(msgbuf,
 	"Can't create directory <%.500s...>: it would exceed MAXPATHLEN = %d\n",
-	prefix, MAXPATHLEN);
+	    prefix, MAXPATHLEN);
+	} else {
+	    Error(0, 0,
+	"Can't create directory <%.500s...>: it would exceed MAXPATHLEN = %d\n",
+	    prefix, MAXPATHLEN);
+	}
 	return NULL;
     }
 
@@ -1095,16 +1237,29 @@ char *prefix, *hostname, *file;
 /* Creates a directory, including any needed directories leading
  * to it.  Returns -1 on stat/mkdir failure; 0 otherwise.
  * WARNING: doesn't check if final component is a directory.
+ *
+ * *err is returned with an errno value, if there was an error
+ * for which there's a relevant errno.  Otherwise it's set to 0.
+ *
+ * If msgbuf is non-null pointer, then it's *assumed* long enough
+ * for error message (1000 chars is enough), and error messages
+ * are printed into it (without the following perror() value, which
+ * should be added later; otherwise, error messages are Error()'d.
  */
+
 /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
 int
-makedir(dir)
+makedir(dir, err, msgbuf)
 char *dir;	/* path with directories only. */
+int *err;
+char *msgbuf;
 {
     static struct stat st;
     char *q;
     char path[MAXPATHLEN];
 
+    *err = errno;
+
     /* First create directories along way, if necessary */
     strcpy(path, dir);
 
@@ -1122,11 +1277,23 @@ char *dir;	/* path with directories only. */
 	/* Stat directory; if missing, create it */
 	if (stat(path, &st) != 0) {
 	    if (errno != ENOENT) {
-		return Error(1, 0, "Failed to stat directory `%s'\n", path);
-	    } else {
-		if (mkdir(path, 0700) != 0)
-		    return Error(1, 0,
-				"Failed to create directory `%s'\n", path);
+		*err = errno;
+		if (msgbuf) {
+		    sprintf(msgbuf,
+			    	"Failed to stat directory <%.500s>: ", path);
+		} else {
+		    Error(1, 0, "Failed to stat directory <%.500s>: ", path);
+		}
+		return -1;
+	    } else if (mkdir(path, 0700) != 0) {
+		*err = errno;
+		if (msgbuf) {
+		    sprintf(msgbuf,
+			    "Failed to create directory <%.500s>: ", path);
+		} else {
+		    Error(1, 0, "Failed to create directory <%.500s>: ", path);
+		}
+		return -1;
 	    }
 	}
 
@@ -1152,8 +1319,8 @@ char *vardefn;
      * Allocate space for varname, vardefn, and insert into
      * vars table.
      */
-    ENTRY item;
-    char *space;
+    ENTRY item, *found_item;
+    char *keyspace, *dataspace;
     int ln = strlen(varname);
     int lb = strlen(vardefn);
 
@@ -1168,20 +1335,28 @@ char *vardefn;
 	fprintf(stderr, "entering add_variable(\"%s\",\"%s\")\n",
 		varname, vardefn);
 
-    space = malloc(ln+1 + lb+1);
-    if (!space)
+    keyspace = malloc(ln+1);
+    dataspace = malloc(lb+1);
+    if (!keyspace || !dataspace) {
 	return Error(0, 0,
 	    "%tFailed to allocate space for definition of variable `%s'\n",
 	    varname);
+    }
 
-    item.key = space;
-    item.data = space + ln+1;
+    item.key = keyspace;
+    item.data = dataspace;
     strcpy(item.key, varname);
     strcpy(item.data, vardefn);
 
     if (debug > 1)
 	fprintf(stderr, "Adding variable `%s' = `%s'\n", item.key, item.data);
 
+    /* Delete the existing value, if any */
+    if ((found_item = hsearch(item, FIND))) {
+	/* Assume all keys and data allocated via malloc() */
+	free(found_item->key);
+	free(found_item->data);
+    }
     if (!hsearch(item, ENTER))
 	return Error(0, 0,
 	"%tFailed to allocate space for hash-table entry for variable `%s'\n",
@@ -1531,7 +1706,7 @@ char *str;
 /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
 /* re_comp()-style interface to wildmat. */
 /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
-static char *shell_pattern;
+static char *shell_pattern = NULL;
 char *
 shell_compile(s)
 char *s;
@@ -1544,11 +1719,62 @@ char *s;
 /* re_exec()-style interface to wildmat. */
 /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
 int
-shell_compare(s)
-char *s;
+shell_compare(str)
+char *str;
 {
     extern int wildmat P__((char *str, char *pat));
-    return wildmat(s, shell_pattern);
+    int l, Reverse;
+    char *sp;
+
+    /* If the pattern is [[...]] (that is, bounded by doubled
+     * square brackets), then we have a special case:
+     * every character in s must match the glob pattern [...].
+     * For consistency with our normal shell patterns, we allow
+     * ^[[...]] to invert the overall sense of match, but note
+     * that the effect is the same as [[^...]].
+     */
+    Reverse = 0;
+    sp = shell_pattern;
+    if (*sp == '^') {
+	Reverse = 1;
+	sp++;
+    }
+    l = strlen(sp);
+    if (sp[0] == '[' && sp[1] == '[' && sp[l-1] == ']' && sp[l-2] == ']') {
+	/*
+	 * it's the special case...
+	 */
+	int last, matched, reverse;
+	char *p;
+	
+	for ( ; *str; str++) {
+	    p = sp+1;			/* skip leading "[" of "[[" */
+	    if (p[1] == '^') {
+		reverse = !Reverse;	/* Inverted character class. */
+		p++;
+	    } else {
+		reverse = Reverse;
+	    }
+	    matched = 0;
+	    if (p[1] == ']' || p[1] == '-') {
+		if (*++p == *str) {
+		    matched = 1;
+		}
+	    }
+	    for (last = *p; *++p && *p != ']'; last = *p) {
+		if (*p == '-' && p[1] != ']'
+		    ? *str <= *++p && *str >= last : *str == *p) {
+		    matched = 1;
+		}
+	    }
+	    if (matched == reverse) {
+		return 0;		/* fail */
+	    }
+	}
+	return 1;
+    } else {
+	return wildmat(str, shell_pattern);
+    }
 }
 
 /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
